;***************************************************************************
;*		  HSV  RGB
;***************************************************************************

; :
.def	HSV_H	= r1	; HSV_H    byte[0..255] (Hue)        
.def	HSV_S	= r12	; HSV_S    byte[0..255] (Saturation) 
.def	HSV_V	= r13	; HSV_B    byte[0..255] (Brightness) 
;_HSV  byte 1

; :
.def	HSV_V1	= r7	;  v1
.def	HSV_V2	= r8	;  v2
.def	HSV_V3	= r9	;  v3
.def	HSV_i	= r10	;  
.def	HSV_f	= r11	;  
;.def	NULL	= r2
;.def	FULL	= r3

; :
.def	rgb_r	= r4	; RGB_R    byte[0..255]
.def	rgb_g	= r5	; RGB_G    byte[0..255]
.def	rgb_b	= r6	; RGB_B    byte[0..255]

;   ,R14,R15, R16, R17, R18, R19, Z



/* 
;============================================================
; Nur zu Testzwecken
	ldi   	r16, 127		; [0..255] 
	ldi   	r17, 255		;  [0..255]
	ldi   	r18, 255		; [0..255]
	mov   	hsv_h, r16
	mov   	hsv_s, r17
	mov   	hsv_v, r18
; Ende Testload HSV-Werte

    rcall 	HSV_2_RGB
*/


;-------------------------------------------------------------------
;HSB -> RGB Wandler, in AVR-Assembler, fr ATiny und ATmega
;Kurzfeatures: (Details und ausfhrliche Erluterungen im Quelltext)
;-------------------------------------------------------------------
;*Codegrsse nur ca.300 Words auf Atiny, ca.280 auf ATmega
;*Eingang: 3Byte im SRAM (H-S-B)
;*Ausgang: 3Byte im SRAM (R-G-B)
;*Belegt nur 6Byte SRAM und wenig Stack
;*Bentigt keine dauerhaften Register
;*komplett in AVR-Assembler, bentigt keine Zusatzmodule, o..
;*vollstndig dokumentiert, praktisch zeilenweise

;-------------------
;EIN-/AUSGANGSDATEN:
;-------------------
;Die Routine erwartet 3 Bytes Eingangsdaten im SRAM:
;H byte[0..192] (Hue)        Farbwinkel 0=rot, 64=grn, 128=blau
;S byte[0..100%] (Saturation) Sttigung  0%=grau, 50%=wenig Sttigung, 100%=reine Farbe
;B byte[0..100%] (Brightness) Helligkeit 0%=keine 100%=volle Helligkeit

;Es werden 3 Bytes zurckgegeben im SRAM:
;R byte[0..100]
;G byte[0..100]
;B byte[0..100]
;==============================================

HSV_2_RGB_max:
		lds	HSV_H,_HSV					;
HSV_2_RGB_sv:
		mov	HSV_V,FULL					;
HSV_2_RGB_s:
		mov HSV_S,FULL					;
HSV_2_RGB:     ;===============================
               ;sonderfall sttigung S=0? 
               tst   hsv_s                 ;Sttigung=0?
               brne  hsb2rgb1              ;nein: dann normal weiter
               mov   rgb_r,hsv_v           ;ja: RGB alle auf V-Wert setzen
               mov   rgb_g,hsv_v           ;ja: RGB alle auf V-Wert setzen
               mov   rgb_b,hsv_v           ;ja: RGB alle auf V-Wert setzen
               ret

               ;1. zwischenwert:  (byte)v1 =(256-s); v1=(V*v1) /256
hsb2rgb1:      ldi		r16,255
               sub		r16,hsv_s			; r16 = (256-S)
               mov		r17,hsv_v			; r17 = V
               rcall	HSV_MULT			; r17(L):r18(H) = V*(256-S)
;               rcall	HSV_Div256		; zl = (V*(256-S)) / 256
               mov		HSV_V1,r18		; Zwischenwert v1 sichern 

               ;===============================
               ;2. zwischenwert: (byte)v2 = (V*((10710-(S*f))/42)) /256
               mov		r16,hsv_h
               rcall	HSV_Div43			; H/43
               mov		HSV_i,r18			; Farbvollwinkel
			   mov		HSV_f,r16		; Farbwinkel
			   mov		r17,hsv_s
               rcall	HSV_MULT			; r17(L):r18(H) = (S*f)
               ldi		r16,byte1(10710)
			   ldi		r19,byte2(10710)
			   sub		r16,r17				; r17(L):r18(H) = 10710 - (S*f)
			   sbc		r19,r18
hsb2rgb1a:	   mov		r17,r19
			   rcall	HSV_Div42			; r17 = (10710 - (S*f))/42
			   mov		r16,hsv_v
			   rcall	HSV_MULT			; r17(L):r18(H) = V*((10710 - (S*f))/42)
;			   rcall	HSV_Div256			; r17(L):r18(H) = (V*((10710 - (S*f))/42))/256
hsb2rgb1b:	   mov		HSV_V2,r18		    ; Zwischenwert v2 sichern

               ;===============================
               ;3. zwischenwert: (byte)v3 = ((10710 - (S*(42-f)))/42))/256
               ldi		r16, 42
               sub		r16,hsv_f			; r16=(42-f) (f steht in r17)
			   mov		r17,hsv_s			; r17 =S
               rcall	HSV_MULT			; r17(L):r18(H)= (S*(42-f))
			   ldi		r16, low(10710)
			   ldi		r19,high(10710)
			   sub		r16, r17			; r17(L):r18(H)= 10710 - (S*(42-f))
			   sbc		r19, r18
hsb2rgb2a:	   mov		r17, r19
			   rcall	HSV_Div42			; r17(L)= (10710 - (S*(42-f)))/42
			   mov		r16, hsv_v
			   rcall	HSV_MULT			; r17(L):r18(H)= V*((10710 - (S*(42-f)))/42)
;			   rcall	HSV_Div256			; r17(L):r18(H)= (V*((10710 - (S*(42-f)))/42))/256
hsb2rgb2b:	   mov		HSV_V3,r18			; Zwischenwert v3 sichern

               ;===============================
               ;4. Farbwerte zu RGB speichern
			   ldi		zl, low(switch_rgb)
			   ldi		zh,high(switch_rgb)
			   add		zl,hsv_i
			   adc		zh,NULL
			   ijmp
               ;===============================


;==================================================================================================
;***** Subroutine Register Variables

;.def	r16	=r16		;multiplicand
;.def	r17	=r17		;multiplier
;.def	r17	=r17		;result Low byte
;.def	r18	=r18		;result High byte


;    MUL
HSV_MULT:		clr		r18		;clear result High byte
				lsr		r17		;shift multiplier
	
				brcc	hsv_mult1	;if carry set
				add		r18, r16	;    add multiplicand to result High byte
hsv_mult1:		ror		r18		;shift right result High byte 
				ror		r17		;rotate right result L byte and multiplier

				brcc	hsv_mult2	;if carry set
				add		r18, r16	;    add multiplicand to result High byte
hsv_mult2:		ror		r18		;shift right result High byte 
				ror		r17		;rotate right result L byte and multiplier

				brcc	hsv_mult3	;if carry set
				add		r18, r16	;    add multiplicand to result High byte
hsv_mult3:		ror		r18		;shift right result High byte 
				ror		r17		;rotate right result L byte and multiplier

				brcc	hsv_mult4	;if carry set
				add		r18, r16	;    add multiplicand to result High byte
hsv_mult4:		ror		r18		;shift right result High byte 
				ror		r17		;rotate right result L byte and multiplier
				brcc	hsv_mult5	;if carry set
				add		r18, r16	;    add multiplicand to result High byte
hsv_mult5:		ror		r18		;shift right result High byte 
				ror		r17		;rotate right result L byte and multiplier
				brcc	hsv_mult6	;if carry set
				add		r18, r16	;    add multiplicand to result High byte
hsv_mult6:		ror		r18		;shift right result High byte 
				ror		r17		;rotate right result L byte and multiplier
				brcc	hsv_mult7	;if carry set
				add		r18, r16	;    add multiplicand to result High byte
hsv_mult7:		ror		r18		;shift right result High byte 
				ror		r17		;rotate right result L byte and multiplier
				brcc	hsv_mult8	;if carry set
				add		r18, r16	;    add multiplicand to result High byte
hsv_mult8:		ror		r18		;shift right result High byte 
				ror		r17		;rotate right result L byte and multiplier
				ret


;==================================================================================================
;    r16	; Divident
;    r17	; Divisor
;    r18	; result
;    r19	; rest

HSV_Div43:	ldi		r17, 43
			clr		r18			; Ergebnis initialisieren
			ldi		r19, 1		; Teilbarkeit durch 1 annehmen
			tst		r17
hsv_div43_1:
			brmi	hsv_div43_2		; Den Divisor linksb?ndig ? -> JA
			lsl		r19			; Zwischenergebnis und
			lsl		r17			; Divisor einen Schritt nach links
			rjmp	hsv_div43_1		; ... und nochmal pruefen
hsv_div43_2:
			cp		r16, r17	; divident >= divisor
			brlo	hsv_div43_3		; -> nein
			add		r18, r19	; Ergebnis vergr??ern und
			sub		r16, r17	; Dividenten verkleinern
hsv_div43_3:
			lsr		r17			; Divisor und Zwischenergebnis
			lsr		r19			; einen Schritt nach rechts
			brne	hsv_div43_2		; r19!=0 ? ->JA
		ret							; Ergebnis in ergebnis; Rest in divident !

;==================================================================================================
;    r16 ; LSB Divident
;    r17 ; MSB Divident
;    r18 ; Hifsregister
.def 	rel  = r14 ; LSB Ergebnis
.def 	reh  = r15 ; MSB Ergebnis

HSV_Div42:	; Divieren von r17:r16 durch rd2

			clr 	r18 ; Leere Hilfsregister
			clr 	r15  ; Leere Ergebnisregister
			clr 	r14  ; (Ergebnisregister dient auch als
			inc 	r14  ; Zhler bis 16! Bit 1 auf 1 setzen)
;
; Hier beginnt die Divisionsschleife
;
hsv_Div42a:
			clc      			; Carry-Bit leeren
			rol 	r16 		; nchsthheres Bit des Dividenten
			rol 	r17 		; in das Hilfsregister rotieren
			rol 	r18 		; (entspricht Multipliklation mit 2)
			brcs 	hsv_Div42b 	; Eine 1 ist herausgerollt, ziehe ab
			cpi 	r18,42 		; Divisionsergebnis 1 oder 0?
			brcs 	hsv_Div42c  ; berspringe Subtraktion, wenn kleiner
hsv_Div42b:
			subi 	r18,42		; Subtrahiere Divisor
			sec      			; Setze carry-bit, Ergebnis ist eine 1
			rjmp 	hsv_Div42d  ; zum Schieben des Ergebnisses
hsv_Div42c:
			clc      			; Lsche carry-bit, Ergebnis ist eine 0
hsv_Div42d:
			rol 	r14  		; Rotiere carry-bit in das Ergebnis
			rol 	r15
			brcc 	hsv_Div42a  ; solange Nullen aus dem Ergebnis
	            				; rotieren: weitermachen
; Ende der Division erreicht
			mov 	r17,r14
			ret


;==================================================================================================
switch_rgb:
	rjmp	phi_rot
	rjmp	phi_gelb
	rjmp	phi_gruen
	rjmp	phi_tuerkis
	rjmp	phi_blau
	rjmp	phi_violett

phi_rot:
		mov		rgb_r, hsv_v
		mov		rgb_g, hsv_v3
		mov		rgb_b, hsv_v1
	ret

phi_gelb:
		mov		rgb_r, hsv_v2
		mov		rgb_g, hsv_v
		mov		rgb_b, hsv_v1
	ret

phi_gruen:
		mov		rgb_r, hsv_v1
		mov		rgb_g, hsv_v
		mov		rgb_b, hsv_v3
	ret

phi_tuerkis:
		mov		rgb_r, hsv_v1
		mov		rgb_g, hsv_v2
		mov		rgb_b, hsv_v
	ret

phi_blau:	
		mov		rgb_r, hsv_v3
		mov		rgb_g, hsv_v1
		mov		rgb_b, hsv_v
	ret

phi_violett:
		mov		rgb_r, hsv_v
		mov		rgb_g, hsv_v1
		mov		rgb_b, hsv_v2
	ret



